<?php
// +---------------------------------------------------------------------------+
// | This phpblock Blog Block for Geeklog                                      |
// +---------------------------------------------------------------------------+
// | Blog用の各種ブロックを表示（PHPブロックや静的ページなどへ表示できる）     |
// |   phpblock_blogblock.php   Version: 2.0.7                                 |
// +---------------------------------------------------------------------------+
// | Copyright (C) 2008-2013 by the following authors:                         |
// |                                                                           |
// | Authors: Hiroshi Sakuramoto - hiro AT winkey DOT jp                       |
// |                                                                           |
// +---------------------------------------------------------------------------+
// | Reference: phpblock_newentries                                            |
// | Reference: phpblock_category                                              |
// | Reference: phpblock_newcomments                                           |
// | Reference: phpblock_archives                                              |
// | Reference: phpblock_storystats                                            |
// | --                                                                        |
// | Reference: phpautotags_newentries                                         |
// | Reference: phpautotags_storystats                                         |
// +---------------------------------------------------------------------------+


/* 最新記事の表示用PHPブロック関数 - phpblock_newentries

   表示例
     New Entries
       アンケートにご協力くだ..
       Geeklogへようこそ
*/
//  $numrows: 表示件数 (デフォルト5件)
//  $length:  表示するタイトルの文字数 (デフォルト24文字)
//  $introlength:  表示するイントロ記事の文字数 (デフォルト72文字)
//  $trimstr: テキストを文字数で丸めた場合に末尾に付加する文字列(デフォルト"..")
//  $title_postmsg: タイトルの後に付加する文字列 (デフォルト"")
//  $exclude: 表示制限する話題 (デフォルトなし) 指定例: array('General','Geeklog'); 
//  $exclude_flg: $excludeで指定した話題を表示する/表示しないを切り替える (デフォルト=表示しない)
//  $list_flg: 最新記事を<dl><dt>のリストで表示する/表示しないを切り替える (デフォルト=表示しない)
//  $newentries_flg: 最新情報を<ul><li>のリストで表示する/表示しないを切り替える (デフォルト=表示しない)
//  $dateformat: 日付のフォーマット(dateのformat) (デフォルト=""なし)
//  $morestr: 記事へのリンクを表す文字列 (デフォルト="more")
//  $arrowimg: 記事へのリンク文字の左へ出す矢印画像のurl (デフォルト=""なし)
function phpblock_newentries($numrows=5, $length=24, $introlength=0, $trimstr="..", $title_postmsg="", $exclude=null, $exclude_flg=false, $list_flg=false, $newentries_flg=true, $dateformat="", $morestr="more", $arrowimg="")
{
    global $_CONF, $_TABLES;
    $retval = '';
    
    // SQL
    $exc = '';
    if ($exclude_flg == true) {
        if (count($exclude)>0) {
            $exc = "AND (t.tid='" . array_shift($exclude) . "'";
            if (count($exclude)>0) {
                $exc .= count($exclude)>0 ? implode('',preg_replace('/^(.*)$/'," OR t.tid='$0'",$exclude)) : '';
            }
            $exc .= ")";
        }
    } else {
        $exc = count($exclude)>0 ? implode('',preg_replace('/^(.*)$/'," AND t.tid!='$0'",$exclude)) : '';
    }
    
    $sql = "SELECT STRAIGHT_JOIN s.sid, t.tid, s.title, s.date, s.group_id, s.owner_id, ";
    $sql .= "s.perm_owner, s.perm_group, s.perm_members, s.perm_anon, ";
    $sql .= "t.topic, ";
    $sql .= "s.introtext ";
    $sql .= "FROM {$_TABLES['stories']} as s, {$_TABLES['topics']} as t ";
    $sql .= ", {$_TABLES['topic_assignments']} as a ";
    $sql .= "WHERE s.title<>'' AND (s.sid = a.id)  AND (a.tid = t.tid) AND (date <= NOW()) AND (s.draft_flag = 0) ". $exc;
    $sql .= " ORDER BY s.date DESC LIMIT " . $numrows;
    
    // Get DB
    $result = DB_query($sql);
    $nrows = DB_numRows($result);
    
    // Display title of stories.
    if( $nrows>0 ) {
        if( $newentries_flg == true ) { $retval .= '<ul class="newinfo">'; }
        elseif( $list_flg == true ) { $retval .= '<dl>'; }
        else { $retval .= '<div class="blogblock">'; }
        for( $i=1; $i<=$nrows; $i++ ) {
            $A = DB_fetchArray($result);
            $T_ACCESS=SEC_hasTopicAccess($A['tid']);
            $ACCESS=SEC_hasAccess($A['owner_id'], $A['group_id'] ,$A['perm_owner'] ,$A['perm_group'] ,$A['perm_members'] ,$A['perm_anon']);
            
            // date
            $date = "";
            if ($dateformat <> "") {
                $date = date($dateformat, strtotime($A['date']));
            }
            // title trim
            $title =  mb_strimwidth($A['title'], 0, $length, $trimstr);
            if( $list_flg == false && $newentries_flg == false ) {
                if( $introlength>0 ) {
                    // introtext autotag parse
                    $introbuf = PLG_replacetags( $A['introtext'] );
                    // introtext tag delete
                    $tag_regex_ = "[^\"'<>]*(?:\"[^\"]*\"[^\"'<>]*|'[^']*'[^\"'<>]*)*(?:>|(?=<)|$(?!\n))";
                    $introbuf2 = preg_replace("/\<$tag_regex_/i"," ",$introbuf);
        //            $introbuf = _phpblock_blockblog_tag_filter($introbuf);
                    // introtext trim
                    $introtext =  mb_strimwidth($introbuf2, 0, $introlength, $trimstr);
                    
                    $retval .= '<div class="blogblock-content'.$i.'">';
                    if ($date <> "") { $retval .= '<h3>'.$date.'</h3>'; }
                    $retval .= '<h4>'.$title.'</h4>';
                    $retval .= '<p>'.$introtext.'</p>';
                    
                    if ($ACCESS>0 AND $T_ACCESS>0) {
                        $more = (empty($arrowimg)) ? $morestr : '<img src="'.$arrowimg.'" alt=""'.XHTML.'>'.$morestr ;
                        $retval .= '<div class="morelink"><a href="'.COM_buildUrl($_CONF['site_url'].'/article.php?story='.$A['sid']).'">'.$more.'</a></div>'.LB;
                    } else {
                        $retval .= LB;
                    }
                    $retval .= '</div>';
                } else {
                    $retval .= '<div class="blogblock-content'.$i.'">';
                    if ($date <> "") { $retval .= '<h3>'.$date.'</h3>'; }
                    $retval .= '<h4><a href="'.COM_buildUrl($_CONF['site_url'].'/article.php?story='.$A['sid']).'">'.$title.'</a>'.$title_postmsg.'</h4>'.LB;
                    $retval .= '</div>';
                }
            } elseif ( $newentries_flg == true ) {
                if ($ACCESS>0 AND $T_ACCESS>0) {
                    $retval .= '<li class="newinfo">';
                    if ($date <> "") { $date='<time datetime='.strftime('%Y-%m-%dT%H:%M:%S',strtotime($date)).'>'.$date.'</time> &nbsp; '; }
                    $retval .= '<a href="'.COM_buildUrl($_CONF['site_url'].'/article.php?story='.$A['sid']).'">'.$date.$title."</a></li>\n";
                } else {
                    $retval .= '<li class="newinfo">'.$title."</li>\n";
                }
            } else {
                $retval .= '<dt>';
                if ($date <> "") { $date='<time datetime='.strftime('%Y-%m-%dT%H:%M:%S',strtotime($date)).'>'.$date.'</time> &nbsp; '; }
                $retval .= '<a href="'.COM_buildUrl($_CONF['site_url'].'/article.php?story='.$A['sid']).'">'.$date.$title.'</a>'.$title_postmsg.LB;
                $retval .= '</dt>';
            }
        }
        if( $list_flg == false && $newentries_flg == false ) { $retval .= '</div>'; }
        elseif( $newentries_flg == true ) { $retval .= '</ul>'; }
        else { $retval .= '</dl>'; }
    }
    return $retval;
}


/* 記事カテゴリの表示用PHPブロック関数 - phpblock_category
       ( COM_showTopicsと同じ )
   表示例
     Category
       General News (3)
       Geeklog (1)
*/
function phpblock_category()
{
    global $topic;
    $retval = '';
    $retval .= COM_showTopics($topic);
    if (!empty($retval)) {
        $retval = '<ul>'.$retval.'</ul>';
    }
    return $retval;
}


/* 最新コメントの表示用PHPブロック関数 - phpblock_newcomments
     2件以上だと[]で件数ができます。
   表示例
     Recent Comment
       アンケートにご協力くだ... [+2]
       Geeklogへようこそ
*/
function phpblock_newcomments($limit=15)
{
    global $_CONF, $_TABLES, $_USER, $LANG01;

    $retval = '';

    $stwhere = '';

    if( !empty( $_USER['uid'] ))
    {
        $stwhere .= "({$_TABLES['stories']}.owner_id IS NOT NULL AND {$_TABLES['stories']}.perm_owner IS NOT NULL) OR ";
        $stwhere .= "({$_TABLES['stories']}.group_id IS NOT NULL AND {$_TABLES['stories']}.perm_group IS NOT NULL) OR ";
        $stwhere .= "({$_TABLES['stories']}.perm_members IS NOT NULL)";
    }
    else
    {
        $stwhere .= "({$_TABLES['stories']}.perm_anon IS NOT NULL)";
    }
    $sql = "SELECT DISTINCT COUNT(*) AS dups, type, {$_TABLES['stories']}.title, {$_TABLES['stories']}.sid, max({$_TABLES['comments']}.date) AS lastdate FROM {$_TABLES['comments']} LEFT JOIN {$_TABLES['stories']} ON (({$_TABLES['stories']}.sid = {$_TABLES['comments']}.sid)" . COM_getPermSQL( 'AND', 0, 2, $_TABLES['stories'] ) . " AND ({$_TABLES['stories']}.draft_flag = 0) AND ({$_TABLES['stories']}.commentcode = 0)" . $topicsql . COM_getLangSQL( 'sid', 'AND', $_TABLES['stories'] ) . ") WHERE ((({$stwhere}))) GROUP BY {$_TABLES['comments']}.sid,type, {$_TABLES['stories']}.title, {$_TABLES['stories']}.title, {$_TABLES['stories']}.sid ORDER BY 5 DESC LIMIT " . $limit;

    $result = DB_query( $sql );

    $nrows = DB_numRows( $result );

    if( $nrows > 0 )
    {
        $newcomments = array();

        for( $x = 0; $x < $nrows; $x++ )
        {
            $A = DB_fetchArray( $result );

            if(( $A['type'] == 'article' ) || empty( $A['type'] ))
            {
                $urlstart = '<a href="' . COM_buildUrl( $_CONF['site_url']
                    . '/article.php?story=' . $A['sid'] ) . '#comments' . '"';
            }

            $title = COM_undoSpecialChars( stripslashes( $A['title'] ));
            $titletouse = COM_truncate( $title, $_CONF['title_trim_length'],
                                        '...' );

            if( $title != $titletouse )
            {
                $urlstart .= ' title="' . htmlspecialchars( $title ) . '">';
            }
            else
            {
                $urlstart .= '>';
            }

            $acomment = str_replace( '$', '&#36;', $titletouse );
            $acomment = str_replace( ' ', '&nbsp;', $acomment );

            if( $A['dups'] > 1 )
            {
                $acomment .= ' [+' . $A['dups'] . ']';
            }

            $newcomments[] = $urlstart . $acomment . '</a>';
        }

        $retval .= COM_makeList( $newcomments, 'list-new-comments' );
    }
    else
    {
        $retval .= $LANG01[86] . '<br'.XHTML.'>' . LB;
    }
    
    return $retval;
}


/* 過去月間記事の表示用PHPブロック関数 - phpblock_archives
     24ヶ月分までリンク作成(limitで変更可能)
   表示例
     Archives
       2008/07 (3)
       2008/06 (1)
       2008/05 (2)
*/
function phpblock_archives($limit=24)
{
    global $_TABLES, $_CONF;
    $mycaljp_function = "phpblock_mycaljp";
    $mycal_function = "phpblock_mycal";
    $modname = '/mycal';
    
    $retval = '';
    
    if( ! function_exists($mycaljp_function) && ! function_exists($mycal_function) ) {
        return $retval;
    }
    
//    $sql = "SELECT DISTINCT MONTH(date) as month,UNIX_TIMESTAMP(date) AS day,YEAR(date) as year,count(*) AS count FROM {$_TABLES['stories']} WHERE (date <= NOW()) AND (date >= (date_sub(NOW(), INTERVAL 12 MONTH))) AND (draft_flag = 0)" . COM_getTopicSQL('AND', 1) . " GROUP BY YEAR(date),MONTH(date) ORDER BY date DESC LIMIT " . $limit;
    $sql = "SELECT DISTINCT MONTH(date) as month,UNIX_TIMESTAMP(date) AS day,YEAR(date) as year,count(*) AS count FROM {$_TABLES['stories']}, {$_TABLES['topic_assignments']} AS ta WHERE sid=ta.id AND (date <= NOW()) AND (draft_flag = 0)" . COM_getTopicSQL('AND', 1, 'ta') . COM_getLangSQL('sid', 'AND') . " GROUP BY YEAR(date),MONTH(date) ORDER BY date DESC LIMIT " . $limit;
    $result = DB_query( $sql );
    $nrows = DB_numRows( $result );
    
    if( $nrows > 0 ) {
        $retval .= '<ul class="archives">';
        if( function_exists($mycaljp_function) ) {
            $modname = '/mycaljp';
        }
        for( $i = 0; $i < $nrows; $i++ ) {
            $A = DB_fetchArray( $result );
            $day = strftime( '%Y/%m', $A['day'] );
            $count = COM_numberFormat( $A['count'] );
            $startday = strftime( '%Y-%m-01', $A['day'] );
            $endday = strftime( '%Y-%m-31', $A['day'] );
            $url = $_CONF['site_url']
                ."$modname/calsearch.php?datestart=$startday&dateend=$endday&type=stories&mode=search";
            $retval .= '<li class="archives"><a href="'.$url.'">' . $day .' ('.$count.')'. '</a></li>' . LB;
        }
        $retval .= '</ul>';
    }
    return $retval;
}


/* 人気記事の表示用PHPブロック関数 - phpblock_storystats
     人気記事の上位10件を表示(limitで変更可能)

   表示例
     記事（上位10件）

     記事のタイトル              |件
     ----------------------------+------
     Geeklogへようこそ           |10
     アンケートにご協力ください  |4
*/
//  $limit: 表示件数 (デフォルト10件)
//  $title: 表示するタイトル (デフォルト'')
//  $fld01: 表示する一覧のヘッダ項目１のタイトル (デフォルト'')
//  $fld02: 表示する一覧のヘッダ項目２のタイトル  (デフォルト'')
//  $notmsg: 記事が0件の場合の表示メッセージ (デフォルト'')
function phpblock_storystats($limit=10, $title='', $fld01='', $fld02='', $notmsg='')
{
    global $_CONF, $_TABLES, $_USER, $LANG10;
    
    $retval = '';
    
    $title = empty($title) ? $LANG10[7] : $title;
    $fld01 = empty($fld01) ? $LANG10[8] : $fld01;
    $fld02 = empty($fld02) ? $LANG10[9] : $fld02;
    $notmsg = empty($notmsg) ? $LANG10[10] : $notmsg;

    // Detailed story statistics
    $result = DB_query("SELECT sid,title,hits FROM {$_TABLES['stories']} WHERE (draft_flag = 0) AND (date <= NOW()) AND (Hits > 0)" . COM_getPermSQL ('AND') . $topicsql . " ORDER BY hits DESC LIMIT " . $limit);
    $nrows  = DB_numRows($result);

    if ($nrows > 0) {
        require_once $_CONF['path_system'] . 'lib-admin.php';
        $header_arr = array(
            array('text' => $fld01, 'field' => 'sid', 'header_class' => 'stats-header-title'),
            array('text' => $fld02, 'field' => 'hits', 'header_class' => 'stats-header-count', 'field_class' => 'stats-list-count'),
        );
        $data_arr = array();
        $text_arr = array('has_menu'     =>  false,
                          'title'        => $title,
        );

        for ($i = 0; $i < $nrows; $i++) {
            $A = DB_fetchArray($result);
            $A['title'] = stripslashes(str_replace('$','&#36;',$A['title']));
            $A['sid'] = COM_createLink($A['title'],  COM_buildUrl ($_CONF['site_url']
                      . "/article.php?story={$A['sid']}"));
            $A['hits'] = COM_NumberFormat ($A['hits']);
            $data_arr[$i] = $A;

        }
        $retval .= ADMIN_simpleList("", $header_arr, $text_arr, $data_arr);
    } else {
        $retval .= COM_startBlock($title);
        $retval .= $notmsg;
        $retval .= COM_endBlock();
    }
    
    return $retval;
}


/* 自動タグ [newentories:] 関数 - phpautotags_newentries
     autotags PHP関数 - newentries
*/
function phpautotags_newentries($p1, $p2, $fulltag)
{
    if( empty($p1) ) { return ''; }
    // default
    $row = $p1;
    $len = 24;
    $introlen = 72;
    $trimstr = '..';
    $title_postmsg = "";
    $exclude = null;
    $exclude_flg = false;
    $list_flg = false;
    $newentries_flg = false;
    $dateformat = "";
    $morestr = "more";
    $arrowimg = "";
    
    $px = explode (' ', trim ($p2));
    foreach ($px as $part) {
        if (substr ($part, 0, 4) == 'row:') {
            $a = explode (':', $part);
            $row = $a[1];
        }
        if (substr ($part, 0, 4) == 'len:') {
            $a = explode (':', $part);
            $len = $a[1];
        }
        if (substr ($part, 0, 9) == 'introlen:') {
            $a = explode (':', $part);
            $introlen = $a[1];
        }
        if (substr ($part, 0, 8) == 'trimstr:') {
            $a = explode (':', $part);
            $trimstr = $a[1];
        }
        if (substr ($part, 0, 13) == 'titlepostmsg:') {
            $a = explode (':', $part);
            $title_postmsg = $a[1];
        }
        if (substr ($part, 0, 12) == 'exclude_flg:') {
            $a = explode (':', $part);
            $exclude_flg = $a[1];
        }
        if (substr ($part, 0, 8) == 'exclude:') {
            $a = explode (':', $part);
            $exclude = explode (',', $a[1]);
        }
        if (substr ($part, 0, 9) == 'list_flg:') {
            $a = explode (':', $part);
            $list_flg = $a[1];
        }
        if (substr ($part, 0, 15) == 'newentries_flg:') {
            $a = explode (':', $part);
            $newentries_flg = $a[1];
        }
        if (substr ($part, 0, 11) == 'dateformat:') {
            $a = explode (':', $part);
            $dateformat = $a[1];
        }
        if (substr ($part, 0, 8) == 'morestr:') {
            $a = explode (':', $part);
            $morestr = $a[1];
        }
        if (substr ($part, 0, 9) == 'arrowimg:') {
            $a = explode (':', $part);
            $arrowimg = $a[1];
        }
    }
    return phpblock_newentries($row, $len, $introlen, $trimstr, $title_postmsg, $exclude, $exclude_flg, $list_flg, $newentries_flg, $dateformat, $morestr, $arrowimg);
}

/* 自動タグ [storystats:] 関数 - phpautotags_storystats
     autotags PHP関数 - storystats
*/
function phpautotags_storystats($p1, $p2, $fulltag)
{
    if( empty($p1) ) { return ''; }
    // default
    $limit = $p1;
    $title = '';
    $fld01 = '';
    $flg02 = '';
    $notmsg = '';
    
    $px = explode (' ', trim ($p2));
    foreach ($px as $part) {
        if (substr ($part, 0, 6) == 'title:') {
            $a = explode (':', $part);
            $title = $a[1];
        }
        if (substr ($part, 0, 6) == 'fld01:') {
            $a = explode (':', $part);
            $fld01 = $a[1];
        }
        if (substr ($part, 0, 6) == 'fld02:') {
            $a = explode (':', $part);
            $fld02 = $a[1];
        }
        if (substr ($part, 0, 7) == 'notmsg:') {
            $a = explode (':', $part);
            $notmsg = $a[1];
        }
    }
    return phpblock_storystats($limit, $title, $fld01, $fld02, $notmsg);
}

?>